@@ -46,9 +46,22 @@ def _get_resource_programs(fn, algs):
 
     Helps avoid requiring core information from unused programs.
     """
+    used_progs = _get_used_programs(fn, algs)
     # standard list of programs we always use
     # XXX Need to expose this in a top-level way to allow more multiprocessing
-    used_progs = set(["gatk", "gemini", "bcbio_coverage", "samtools", "snpEff"])
+    for prog in (fn.metadata.get("resources", []) if hasattr(fn, "metadata") else []):
+        if prog in used_progs:
+            yield prog
+
+def _get_ensure_functions(fn, algs):
+    used_progs = _get_used_programs(fn, algs)
+    for prog in (fn.metadata.get("ensure", {}).keys() if hasattr(fn, "metadata") else []):
+        if prog in used_progs:
+            yield fn.metadata["ensure"][prog]
+
+def _get_used_programs(fn, algs):
+    used_progs = set(["gatk", "gemini", "bcbio_coverage", "samtools",
+                      "snpEff", "cufflinks", "picard"])
     for alg in algs:
         # get aligners used
         aligner = alg.get("aligner")
@@ -63,9 +76,8 @@ def _get_resource_programs(fn, algs):
                 used_progs.add(vc)
     if config_utils.use_vqsr(algs):
         used_progs.add("gatk-vqsr")
-    for prog in (fn.metadata.get("resources", []) if hasattr(fn, "metadata") else []):
-        if prog in used_progs:
-            yield prog
+    return used_progs
+
 
 def _str_memory_to_gb(memory):
     val = float(memory[:-1])
@@ -76,6 +88,7 @@ def _str_memory_to_gb(memory):
         assert units.lower() == "g", "Unexpected memory units: %s" % memory
     return val
 
+
 def _get_prog_memory(resources):
     """Get expected memory usage, in Gb per core, for a program from resource specification.
     """
@@ -88,6 +101,7 @@ def _get_prog_memory(resources):
         out = _str_memory_to_gb(memory)
     return out
 
+
 def _scale_cores_to_memory(cores, mem_per_core, sysinfo, system_memory):
     """Scale multicore usage to avoid excessive memory usage based on system information.
     """
@@ -95,7 +109,7 @@ def _scale_cores_to_memory(cores, mem_per_core, sysinfo, system_memory):
     if "cores" not in sysinfo:
         return cores, total_mem
     cores = min(cores, int(sysinfo["cores"]))
-    total_mem = min(total_mem, float(sysinfo["memory"]) - system_memory)
+    total_mem = min(float(total_mem), float(sysinfo["memory"]) - system_memory)
     cores = max(1, min(cores, int(math.floor(float(total_mem) / mem_per_core))))
     return cores, total_mem
 
@@ -132,16 +146,26 @@ def find_job_resources(fns, parallel, items, sysinfo, config, multiplier=1,
     for fn in fns:
         for prog in _get_resource_programs(fn, algs):
             resources = config_utils.get_resources(prog, config)
-            cores = resources.get("cores")
-            if cores:
-                all_cores.append(cores)
+            cores = resources.get("cores", 1)
             memory = _get_prog_memory(resources)
+            all_cores.append(cores)
             if memory:
                 all_memory.append(memory)
+            logger.debug("{prog} requests {cores} cores and {memory}g "
+                         "memory for each core.".format(**locals()))
+
     cores_per_job = max(all_cores)
     if max_multicore:
         cores_per_job = min(cores_per_job, max_multicore)
+    if "cores" in sysinfo:
+        cores_per_job = min(cores_per_job, int(sysinfo["cores"]))
     memory_per_core = max(all_memory)
+
+    # these callbacks make sure the cores and memory meet minimum requirements
+    for fn in fns:
+        for ensure in _get_ensure_functions(fn, algs):
+            cores_per_job, memory_per_core = ensure(cores_per_job, memory_per_core)
+
     total = parallel["cores"]
     if total > cores_per_job:
         num_jobs = total // cores_per_job
@@ -151,11 +175,15 @@ def find_job_resources(fns, parallel, items, sysinfo, config, multiplier=1,
         memory_per_job = "%.1f" % (memory_per_core + system_memory)
         num_jobs = _scale_jobs_to_memory(num_jobs, memory_per_core, sysinfo)
     else:
-        cores_per_job, memory_per_job = _scale_cores_to_memory(cores_per_job, memory_per_core, sysinfo,
+        cores_per_job, memory_per_job = _scale_cores_to_memory(cores_per_job,
+                                                               memory_per_core, sysinfo,
                                                                system_memory)
     # do not overschedule if we don't have extra items to process
     num_jobs = min(num_jobs, len(items) * multiplier)
     JobResources = collections.namedtuple("JobResources", "num_jobs cores_per_job memory_per_job")
+    logger.debug("Configuring %d jobs to run, using %d cores each with %sg of "
+                 "memory reserved for each job" % (num_jobs, cores_per_job,
+                                                   str(memory_per_job)))
     return JobResources(num_jobs, cores_per_job, str(memory_per_job))
 
 cur_num = 0
