@@ -1,8 +1,6 @@
-"""Run distributed tasks using the Celery distributed task queue.
-
-http://celeryproject.org/
+"""Run distributed tasks in parallel using IPython or joblib on multiple cores.
 """
-
+import functools
 import os
 import sys
 import time
@@ -17,7 +15,7 @@ except ImportError:
 
 from bcbio import utils
 from bcbio.distributed import ipython
-from bcbio.log import logger
+from bcbio.log import logger, setup_local_logging
 from bcbio.provenance import diagnostics, system
 
 def parallel_runner(parallel, dirs, config, config_file=None):
@@ -31,6 +29,7 @@ def parallel_runner(parallel, dirs, config, config_file=None):
         imodule = parallel.get("module", "bcbio.distributed")
         sysinfo = system.get_info(dirs, parallel)
         if parallel["type"].startswith("messaging"):
+            raise NotImplementedError("Messaging parallelization no longer supported")
             task_module = "{base}.tasks".format(base=imodule)
             runner_fn = runner(task_module, dirs, config, config_file)
             return runner_fn(fn_name, items)
@@ -41,19 +40,57 @@ def parallel_runner(parallel, dirs, config, config_file=None):
             fn = getattr(__import__("{base}.multitasks".format(base=imodule),
                                     fromlist=["multitasks"]),
                          fn_name)
-            jobr = ipython.find_job_resources([fn], parallel, items, sysinfo, config,
-                                              parallel.get("multiplier", 1),
-                                              max_multicore=int(sysinfo["cores"]))
-            items = [ipython.add_cores_to_config(x, jobr.cores_per_job) for x in items]
-            if joblib is None:
-                raise ImportError("Need joblib for multiprocessing parallelization")
-            out = []
-            for data in joblib.Parallel(jobr.num_jobs)(joblib.delayed(fn)(x) for x in items):
-                if data:
-                    out.extend(data)
-            return out
+            return run_multicore(fn, items, config, parallel["cores"])
     return run_parallel
 
+def zeromq_aware_logging(f):
+    """Ensure multiprocessing logging uses ZeroMQ queues.
+
+    ZeroMQ and local stdout/stderr do not behave nicely when intertwined. This
+    ensures the local logging uses existing ZeroMQ logging queues.
+    """
+    @functools.wraps(f)
+    def wrapper(*args, **kwargs):
+        config = None
+        for arg in args:
+            if ipython.is_std_config_arg(arg):
+                config = arg
+                break
+            elif ipython.is_nested_config_arg(arg):
+                config = arg["config"]
+                break
+        assert config, "Could not find config dictionary in function arguments."
+        if config.get("parallel", {}).get("log_queue"):
+            handler = setup_local_logging(config, config["parallel"])
+        else:
+            handler = None
+        try:
+            out = f(*args, **kwargs)
+        finally:
+            if handler and hasattr(handler, "close"):
+                handler.close()
+        return out
+    return wrapper
+
+def run_multicore(fn, items, config, cores=None):
+    """Run the function using multiple cores on the given items to process.
+    """
+    if cores is None:
+        cores = config["algorithm"].get("num_cores", 1)
+    parallel = {"type": "local", "cores": cores}
+    sysinfo = system.get_info({}, parallel)
+    jobr = ipython.find_job_resources([fn], parallel, items, sysinfo, config,
+                                      parallel.get("multiplier", 1),
+                                      max_multicore=int(sysinfo["cores"]))
+    items = [ipython.add_cores_to_config(x, jobr.cores_per_job) for x in items]
+    if joblib is None:
+        raise ImportError("Need joblib for multiprocessing parallelization")
+    out = []
+    for data in joblib.Parallel(jobr.num_jobs)(joblib.delayed(fn)(x) for x in items):
+        if data:
+            out.extend(data)
+    return out
+
 def runner(task_module, dirs, config, config_file, wait=True):
     """Run a set of tasks using Celery, waiting for results or asynchronously.
 
