@@ -4,10 +4,8 @@ GATK and Picard based pipelines have specific requirements for
 chromosome order, run group information and other BAM formatting.
 This provides a pipeline to prepare and resort an input.
 """
-import contextlib
 import os
 
-import pysam
 
 from bcbio import bam, broad, utils
 from bcbio.distributed.transaction import file_transaction
@@ -31,12 +29,8 @@ def picard_prep(in_bam, names, ref_file, dirs, config):
 def _fix_rgs_and_sort(in_bam, names, ref_file, runner):
     """Fix input read groups if missing and coordinate sort file.
     """
-    with contextlib.closing(pysam.Samfile(in_bam, "rb")) as in_pybam:
-        has_rgs = "RG" in in_pybam.header.keys()
-    if has_rgs:
-        return runner.run_fn("picard_sort", in_bam, "coordinate")
-    else:
-        return runner.run_fn("picard_fix_rgs", in_bam, names)
+    good_rg_bam = runner.run_fn("picard_fix_rgs", in_bam, names)
+    return runner.run_fn("picard_sort", good_rg_bam, "coordinate")
 
 def _filter_bad_reads(in_bam, ref_file, runner, config):
     """Use GATK filter to remove problem reads which choke GATK and Picard.
