@@ -22,6 +22,8 @@ def update_w_custom(config, lane_info):
                    "variant2": ["SNP calling", "variant", "variant2"]}
     config = copy.deepcopy(config)
     base_name = lane_info.get("analysis")
+    if "algorithm" not in config:
+        config["algorithm"] = {}
     for analysis_type in name_remaps.get(base_name, [base_name]):
         custom = config.get("custom_algorithms", {}).get(analysis_type)
         if custom:
@@ -44,26 +46,37 @@ def load_system_config(config_file):
     """Load bcbio_system.yaml configuration file, handling standard defaults.
 
     Looks for configuration file in default location within
-    final base directory from a standard installation.
+    final base directory from a standard installation. Handles both standard
+    installs (galaxy/bcbio_system.yaml) and docker installs (config/bcbio_system.yaml).
     """
     if not os.path.exists(config_file):
         base_dir = os.path.normpath(os.path.join(os.path.realpath(sys.executable), os.pardir, os.pardir, os.pardir))
+        docker_config = os.path.join(base_dir, "config", config_file)
         test_config = os.path.join(base_dir, "galaxy", config_file)
-        if os.path.exists(test_config):
+        if os.path.exists(docker_config):
+            config_file = docker_config
+        elif os.path.exists(test_config):
             config_file = test_config
         else:
             raise ValueError("Could not find input system configuration file %s, "
                              "including inside standard directory %s" %
                              (config_file, os.path.join(base_dir, "galaxy")))
-    return load_config(config_file), config_file
+    config = load_config(config_file)
+    config["bcbio_system"] = config_file
+    return config, config_file
 
 def load_config(config_file):
     """Load YAML config file, replacing environmental variables.
     """
     with open(config_file) as in_handle:
         config = yaml.load(in_handle)
-
     config = _expand_paths(config)
+    # lowercase resource names, the preferred way to specify, for back-compatibility
+    newr = {}
+    for k, v in config["resources"].iteritems():
+        if k.lower() != k:
+            newr[k.lower()] = v
+    config["resources"].update(newr)
     return config
 
 def _expand_paths(config):
