@@ -9,12 +9,15 @@ import pysam
 
 from bcbio import utils, broad
 from bcbio.log import logger
-from bcbio.bam import callable
+from bcbio.bam import callable, ref
 from bcbio.bam.trim import brun_trim_fastq, trim_read_through
 from bcbio.pipeline.fastq import get_fastq_files, needs_fastq_conversion
 from bcbio.pipeline.alignment import align_to_sort_bam
 from bcbio.pipeline import cleanbam
-from bcbio.variation import recalibrate, vcfutils
+from bcbio.variation import recalibrate
+from bcbio import bam
+from bcbio.bam import fastq
+
 
 def _item_needs_compute(lane_items):
     """Determine if any item needs computing resources to spin up a cluster.
@@ -34,10 +37,19 @@ def process_all_lanes(lanes, run_parallel):
         return [process_lane(x)[0] for x in lanes]
 
 def process_lane(item):
-    """Prepare lanes, potentially splitting based on barcodes.
+    """Prepare lanes, potentially splitting based on barcodes and reducing the
+    number of reads for a test run
     """
+    NUM_DOWNSAMPLE = 10000
     logger.debug("Preparing %s" % item["rgnames"]["lane"])
-    item["files"] = get_fastq_files(item)
+    file1, file2 = get_fastq_files(item)
+    if item.get("test_run", False):
+        if bam.is_bam(file1):
+            file1 = bam.downsample(file1, item, NUM_DOWNSAMPLE)
+        else:
+            file1, file2 = fastq.downsample(file1, file2, item,
+                                            NUM_DOWNSAMPLE, quick=True)
+    item["files"] = (file1, file2)
     return [item]
 
 def trim_lane(item):
@@ -94,7 +106,7 @@ def link_bam_file(orig_file, new_dir):
 def _check_prealigned_bam(in_bam, ref_file, config):
     """Ensure a pre-aligned BAM file matches the expected reference genome.
     """
-    ref_contigs = [c["SN"] for c in vcfutils.ref_file_contigs(ref_file, config)]
+    ref_contigs = [c.name for c in ref.file_contigs(ref_file, config)]
     with contextlib.closing(pysam.Samfile(in_bam, "rb")) as bamfile:
         bam_contigs = [c["SN"] for c in bamfile.header["SQ"]]
     problems = []
@@ -130,13 +142,17 @@ def process_alignment(data):
     elif fastq1 and os.path.exists(fastq1) and fastq1.endswith(".bam"):
         sort_method = config["algorithm"].get("bam_sort")
         bamclean = config["algorithm"].get("bam_clean")
-        if sort_method:
+        if bamclean is True or bamclean == "picard":
+            if sort_method and sort_method != "coordinate":
+                raise ValueError("Cannot specify `bam_clean: picard` with `bam_sort` other than coordinate: %s"
+                                 % sort_method)
+            out_bam = cleanbam.picard_prep(fastq1, data["rgnames"], data["sam_ref"], data["dirs"],
+                                           config)
+        elif sort_method:
             runner = broad.runner_from_config(config)
             out_file = os.path.join(data["dirs"]["work"], "{}-sort.bam".format(
                 os.path.splitext(os.path.basename(fastq1))[0]))
             out_bam = runner.run_fn("picard_sort", fastq1, sort_method, out_file)
-        elif bamclean is True or bamclean == "picard":
-            out_bam = cleanbam.picard_prep(fastq1, data["rgnames"], data["sam_ref"], data["dirs"], config)
         else:
             out_bam = link_bam_file(fastq1, os.path.join(data["dirs"]["work"], "prealign",
                                                          data["rgnames"]["sample"]))
@@ -170,3 +186,4 @@ def _recal_no_markduplicates(data):
     data = recalibrate.prep_recal(data)[0][0]
     data["config"] = orig_config
     return data
+
