@@ -7,6 +7,8 @@ import subprocess
 import lxml.html
 import pybedtools
 import yaml
+from datetime import datetime
+
 # allow graceful during upgrades
 try:
     import matplotlib
@@ -67,19 +69,67 @@ def _run_qc_tools(bam_file, data):
         cur_metrics = qc_fn(bam_file, data, cur_qc_dir)
         metrics.update(cur_metrics)
     metrics["Name"] = data["name"][-1]
+    metrics["Quality format"] = utils.get_in(data,
+                                             ("config", "algorithm",
+                                              "quality_format"),
+                                             "standard").lower()
     return {"qc": qc_dir, "metrics": metrics}
 
 # ## Generate project level QC summary for quickly assessing large projects
 
 def write_project_summary(samples):
     """Write project summary information on the provided samples.
+    write out dirs, genome resources,
+
     """
-    out_file = os.path.join(samples[0][0]["dirs"]["work"], "project-summary.yaml")
+    work_dir = samples[0][0]["dirs"]["work"]
+    out_file = os.path.join(work_dir, "project-summary.yaml")
+    upload_dir = (os.path.join(work_dir, samples[0][0]["upload"]["dir"])
+                  if "dir" in samples[0][0]["upload"] else "")
+    test_run = samples[0][0].get("test_run", False)
+    date = str(datetime.now())
     with open(out_file, "w") as out_handle:
-        yaml.dump([data[0]["summary"]["metrics"] for data in samples if "summary" in data[0]],
-                  out_handle, default_flow_style=False, allow_unicode=False)
+        yaml.dump({"date": date}, out_handle,
+                  default_flow_style=False, allow_unicode=False)
+        if test_run:
+            yaml.dump({"test_run": True}, out_handle, default_flow_style=False,
+                      allow_unicode=False)
+        yaml.dump({"upload": upload_dir}, out_handle,
+                  default_flow_style=False, allow_unicode=False)
+        yaml.dump({"bcbio_system": samples[0][0]["config"].get("bcbio_system", "")}, out_handle,
+                  default_flow_style=False, allow_unicode=False)
+        yaml.dump({"samples": [_save_fields(sample[0]) for sample in samples]}, out_handle,
+                  default_flow_style=False, allow_unicode=False)
     return out_file
 
+def _save_fields(sample):
+    to_save = ["dirs", "genome_resources", "genome_build", "sam_ref", "metadata",
+               "description"]
+    saved = {k: sample[k] for k in to_save if k in sample}
+    if "summary" in sample:
+        saved["summary"] = {"metrics": sample["summary"]["metrics"]}
+        # check if disambiguation was run
+        if "disambiguate" in sample:
+            if utils.file_exists(sample["disambiguate"]["summary"]):
+                disambigStats = _parse_disambiguate(sample["disambiguate"]["summary"])
+                saved["summary"]["metrics"]["Disambiguated %s reads" % str(sample["genome_build"])] = disambigStats[0]
+                saved["summary"]["metrics"]["Disambiguated %s reads" % str(sample["algorithm"]["disambiguate"][0])] = disambigStats[1]
+                saved["summary"]["metrics"]["Disambiguated ambiguous reads"] = disambigStats[2]
+    return saved
+
+def _parse_disambiguate(disambiguatestatsfilename):
+    """Parse disambiguation stats from given file.
+    """
+    disambigStats = [-1 -1 -1]
+    with open(disambiguatestatsfilename, "r") as in_handle:
+        header = in_handle.readline().strip().split("\t")
+        if header == ['sample','unique species A pairs','unique species B pairs','ambiguous pairs']:
+            disambigStatsTemp = in_handle.readline().strip().split("\t")[1:]
+            if len(disambigStatsTemp) == 3:
+                disambigStats = [int(x) for x in disambigStatsTemp]
+    return disambigStats
+
+
 # ## Run and parse read information from FastQC
 
 class FastQCParser:
@@ -87,7 +137,8 @@ class FastQCParser:
         self._dir = base_dir
 
     def get_fastqc_summary(self):
-        ignore = set(["Total Sequences", "Filtered Sequences", "Filename", "File type"])
+        ignore = set(["Total Sequences", "Filtered Sequences",
+                      "Filename", "File type", "Encoding"])
         stats = {}
         for stat_line in self._fastqc_data_section("Basic Statistics")[1:]:
             k, v = stat_line.split("\t")[:2]
@@ -142,7 +193,7 @@ def _run_complexity(bam_file, data, out_dir):
     base, _ = os.path.splitext(os.path.basename(bam_file))
     utils.safe_makedir(out_dir)
     out_file = os.path.join(out_dir, base + ".pdf")
-    df = bcbio.rnaseq.qc.starts_by_depth(bam_file)
+    df = bcbio.rnaseq.qc.starts_by_depth(bam_file, data["config"])
     if not utils.file_exists(out_file):
         with file_transaction(out_file) as tmp_out_file:
             df.plot(x='reads', y='starts', title=bam_file + " complexity")
