@@ -40,6 +40,7 @@ def _get_machine_info(parallel, run_parallel, sys_config):
         # dictionary as switch statement; can add new scheduler implementation functions as (lowercase) keys
         sched_info_dict = {
                             "slurm": _slurm_info,
+                            "torque": _torque_info
                           }
         try:
             return sched_info_dict[parallel["scheduler"].lower()](parallel["queue"])
@@ -62,6 +63,50 @@ def _slurm_info(queue):
     mem = mem.replace('+', '')
     return [{"cores": int(num_cpus), "memory": float(mem) / 1024.0, "name": "slurm_machine"}]
 
+def _torque_info(queue):
+    """Return machine information for a torque job scheduler using pbsnodes.
+
+    To identify which host to use it tries to parse available hosts
+    from qstat -Qf `acl_hosts`. If found, it uses these and gets the
+    first node from pbsnodes matching to the list. If no attached
+    hosts are available, it uses the first host found from pbsnodes.
+    """
+    nodes = _torque_queue_nodes(queue)
+    pbs_out = subprocess.check_output(["pbsnodes"])
+    info = {}
+    for i, line in enumerate(pbs_out.split("\n")):
+        if i == 0 and len(nodes) == 0:
+            info["name"] = line.strip()
+        elif line.startswith(nodes):
+            info["name"] = line.strip()
+        elif info.get("name"):
+            if line.strip().startswith("np = "):
+                info["cores"] = int(line.replace("np = ", "").strip())
+            elif line.strip().startswith("status = "):
+                mem = [x for x in pbs_out.split(",") if x.startswith("totmem=")][0]
+                info["memory"] = float(mem.split("=")[1].rstrip("kb")) / 1048576.0
+                return [info]
+
+def _torque_queue_nodes(queue):
+    """Retrieve the nodes available for a queue.
+
+    Parses out nodes from `acl_hosts` in qstat -Qf and extracts the
+    initial names of nodes used in pbsnodes.
+    """
+    qstat_out = subprocess.check_output(["qstat", "-Qf", queue])
+    hosts = []
+    in_hosts = False
+    for line in qstat_out.split("\n"):
+        if line.strip().startswith("acl_hosts = "):
+            hosts.extend(line.replace("acl_hosts = ", "").strip().split(","))
+            in_hosts = True
+        elif in_hosts:
+            if line.find(" = ") > 0:
+                break
+            else:
+                hosts.extend(line.strip().split(","))
+    return tuple([h.split(".")[0].strip() for h in hosts if h.strip()])
+
 def _combine_machine_info(xs):
     if len(xs) == 1:
         return xs[0]
