@@ -6,20 +6,24 @@ import os
 import sys
 import HTSeq
 import itertools
+import pandas as pd
 
 from bcbio.utils import (which, file_exists, get_in, safe_makedir)
 from bcbio.distributed.transaction import file_transaction
-from bcbio.pipeline.alignment import sam_to_querysort_sam
 from bcbio.provenance import do
+from bcbio.log import logger
+from bcbio import bam
 
 
 def _get_files(data):
-    in_file = _get_sam_file(data)
+#    in_file = _get_sam_file(data)
+    in_file = bam.sort(data["work_bam"], data["config"], order="queryname")
     gtf_file = data["genome_resources"]["rnaseq"]["transcripts"]
     work_dir = data["dirs"].get("work", "work")
     out_dir = os.path.join(work_dir, "htseq-count")
     out_file = os.path.join(out_dir, data['rgnames']['sample']) + ".counts"
-    return in_file, gtf_file, out_file
+    stats_file = os.path.join(out_dir, data['rgnames']['sample']) + ".stats"
+    return in_file, gtf_file, out_file, stats_file
 
 
 def is_countfile(in_file):
@@ -37,7 +41,9 @@ def is_countfile(in_file):
 def _get_sam_file(data):
     in_file = data["work_bam"]
     config = data["config"]
-    return sam_to_querysort_sam(in_file, config)
+    sorted = bam.sort(in_file, config, "queryname")
+    sam = bam.bam_to_sam(sorted, config)
+    return sam
 
 
 def invert_strand(iv):
@@ -54,22 +60,40 @@ def invert_strand(iv):
 class UnknownChrom(Exception):
     pass
 
+def _get_stranded_flag(config):
+    strand_flag = {"unstranded": "no",
+                   "firststrand": "reverse",
+                   "secondstrand": "yes"}
+    stranded = _get_strandedness(config)
+    assert stranded in strand_flag, ("%s is not a valid strandedness value. "
+                                     "Valid values are 'firststrand', 'secondstrand', "
+                                     "and 'unstranded")
+    return strand_flag[stranded]
+
+def _get_strandedness(config):
+    return get_in(config, ("algorithm", "strandedness"), "unstranded").lower()
+
 
 def htseq_count(data):
     """ adapted from Simon Anders htseq-count.py script
     http://www-huber.embl.de/users/anders/HTSeq/doc/count.html
     """
 
-    sam_filename, gff_filename, out_file = _get_files(data)
-    stranded = "no"
+    sam_filename, gff_filename, out_file, stats_file = _get_files(data)
+    stranded = _get_stranded_flag(data["config"])
     overlap_mode = "union"
     feature_type = "exon"
     id_attribute = "gene_id"
     minaqual = 0
 
+
     if file_exists(out_file):
         return out_file
 
+    logger.info("Counting reads mapping to exons in %s using %s as the "
+                    "annotation and strandedness as %s." % (os.path.basename(sam_filename),
+                    os.path.basename(gff_filename), _get_strandedness(data["config"])))
+
     features = HTSeq.GenomicArrayOfSets("auto", stranded != "no")
     counts = {}
 
@@ -108,7 +132,10 @@ def htseq_count(data):
                          % feature_type)
 
     try:
-        read_seq = HTSeq.SAM_Reader(sam_filename)
+        if bam.is_sam(sam_filename):
+            read_seq = HTSeq.SAM_Reader(sam_filename)
+        elif bam.is_bam(sam_filename):
+            read_seq = HTSeq.BAM_Reader(sam_filename)
         first_read = iter(read_seq).next()
         pe_mode = first_read.paired_end
     except:
@@ -233,8 +260,14 @@ def htseq_count(data):
 
     with file_transaction(out_file) as tmp_out_file:
         with open(tmp_out_file, "w") as out_handle:
+            on_feature = 0
             for fn in sorted(counts.keys()):
+                on_feature += counts[fn]
                 out_handle.write("%s\t%d\n" % (fn, counts[fn]))
+
+    with file_transaction(stats_file) as tmp_stats_file:
+        with open(tmp_stats_file, "w") as out_handle:
+            out_handle.write("on_feature\t%d\n" % on_feature)
             out_handle.write("no_feature\t%d\n" % empty)
             out_handle.write("ambiguous\t%d\n" % ambiguous)
             out_handle.write("too_low_aQual\t%d\n" % lowqual)
@@ -242,3 +275,28 @@ def htseq_count(data):
             out_handle.write("alignment_not_unique\t%d\n" % nonunique)
 
     return out_file
+
+def combine_count_files(files, out_file=None):
+    for f in files:
+        assert file_exists(f), "%s does not exist or is empty."
+        assert is_countfile(f), "%s does not seem to be a count file."
+    col_names = [os.path.basename(os.path.splitext(x)[0]) for x in files]
+    if not out_file:
+        out_dir = os.path.join(os.path.dirname(files[0]))
+        out_file = os.path.join(out_dir, "combined.counts")
+
+    if file_exists(out_file):
+        return out_file
+
+    df = pd.io.parsers.read_table(f, sep="\t", index_col=0, header=None,
+                                  names=[col_names[0]])
+    for i, f in enumerate(files):
+        if i == 0:
+            df = pd.io.parsers.read_table(f, sep="\t", index_col=0, header=None,
+                                          names=[col_names[0]])
+        else:
+            df = df.join(pd.io.parsers.read_table(f, sep="\t", index_col=0, header=None,
+                                                  names=[col_names[i]]))
+
+    df.to_csv(out_file, sep="\t", index_label="id")
+    return out_file
