@@ -4,7 +4,7 @@ runs of this step.
 """
 import os
 
-from bcbio import broad, utils
+from bcbio import bam, broad, utils
 from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import config_utils, shared
 from bcbio.provenance import do
@@ -50,7 +50,7 @@ def _piped_input_cl(data, region, tmp_dir, out_base_file, prep_params):
         if not utils.file_exists(sel_file):
             with file_transaction(sel_file) as tx_out_file:
                 cl += ["-o", tx_out_file]
-                do.run(cl, "GATK: PrintReads {0}".format(region), data)
+                do.run_memory_retry(cl, "GATK: PrintReads", data, region=region)
         dup_metrics = "%s-dup.dup_metrics" % os.path.splitext(out_base_file)[0]
         compression = "5" if prep_params["realign"] == "gatk" else "0"
         cl = broad_runner.cl_picard("MarkDuplicates",
@@ -64,7 +64,7 @@ def _piped_input_cl(data, region, tmp_dir, out_base_file, prep_params):
         sel_file = data["work_bam"]
     else:
         raise ValueError("Duplication approach not supported with GATK: %s" % prep_params["dup"])
-    broad_runner.run_fn("picard_index", sel_file)
+    bam.index(sel_file, data["config"])
     return sel_file, " ".join(cl)
 
 def _piped_realign_gatk(data, region, cl, out_base_file, tmp_dir, prep_params):
@@ -78,7 +78,7 @@ def _piped_realign_gatk(data, region, cl, out_base_file, tmp_dir, prep_params):
             pipe = ">" if prep_params["dup"] else "-o"
             cmd = "{cl} {pipe} {tx_out_file}".format(**locals())
             do.run(cmd, "GATK pre-alignment {0}".format(region), data)
-    broad_runner.run_fn("picard_index", pa_bam)
+    bam.index(pa_bam, data["config"])
     dbsnp_vcf = data["genome_resources"]["variation"]["dbsnp"]
     recal_file = realign.gatk_realigner_targets(broad_runner, pa_bam, data["sam_ref"],
                                                 dbsnp=dbsnp_vcf, region=region_to_gatk(region))
@@ -180,7 +180,8 @@ def _piped_bamprep_region_fullpipe(data, region, prep_params, out_file, tmp_dir)
         realign_cmd = _piped_realign_cmd(data, prep_params, tmp_dir)
         cmd = "{extract_recal_cmd} {dedup_cmd} {realign_cmd}  > {tx_out_file}"
         cmd = cmd.format(**locals())
-        do.run(cmd, "Piped post-alignment bamprep {0}".format(region), data)
+        do.run_memory_retry(cmd, "Piped post-alignment bamprep {0}".format(region), data,
+                            region=region)
 
 # ## Shared functionality
 
@@ -223,8 +224,7 @@ def piped_bamprep(data, region=None, out_file=None):
             with utils.curdir_tmpdir() as tmp_dir:
                 _piped_bamprep_region(data, region, out_file, tmp_dir)
         prep_bam = out_file
-    broad_runner = broad.runner_from_config(data["config"])
-    broad_runner.run_fn("picard_index", prep_bam)
+    bam.index(prep_bam, data["config"])
     data["work_bam"] = prep_bam
     data["region"] = region
     return [data]
