@@ -11,7 +11,7 @@ from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import config_utils
 from bcbio.provenance import do, programs
 from bcbio.utils import file_exists, append_stem
-from bcbio.variation import samtools
+from bcbio.variation import freebayes, samtools
 from bcbio.variation.vcfutils import (combine_variant_files, write_empty_vcf,
                                       get_paired_bams)
 
@@ -293,15 +293,36 @@ def _varscan_work(align_bams, ref_file, items, target_regions, out_file):
     # zerocoverage calls; strip these with grep, we're not going to
     # call on them
     remove_zerocoverage = "grep -v -P '\t0\t\t$'"
-    cmd = ("{mpileup} | {remove_zerocoverage} "
-           "| java {jvm_opts} -jar {varscan_jar} mpileup2cns --min-coverage 5 --p-value 0.98 "
-           "  --vcf-sample-list {sample_list} --output-vcf --variants "
-           "> {out_file}")
-    cmd = cmd.format(**locals())
-    do.run(cmd, "Varscan".format(**locals()), None,
-           [do.file_exists(out_file)])
+    # write a temporary mpileup file so we can check if empty
+    mpfile = "%s.mpileup" % os.path.splitext(out_file)[0]
+    with file_transaction(mpfile) as mpfile_tx:
+        cmd = ("{mpileup} | {remove_zerocoverage} > {mpfile_tx}")
+        do.run(cmd.format(**locals()), "mpileup for Varscan")
+    if os.path.getsize(mpfile) == 0:
+        write_empty_vcf(out_file)
+    else:
+        cmd = ("cat {mpfile} "
+               "| java {jvm_opts} -jar {varscan_jar} mpileup2cns --min-coverage 5 --p-value 0.98 "
+               "  --vcf-sample-list {sample_list} --output-vcf --variants "
+               "> {out_file}")
+        do.run(cmd.format(**locals()), "Varscan", None,
+               [do.file_exists(out_file)])
     os.remove(sample_list)
+    os.remove(mpfile)
     # VarScan can create completely empty files in regions without
     # variants, so we create a correctly formatted empty file
     if os.path.getsize(out_file) == 0:
         write_empty_vcf(out_file)
+    else:
+        freebayes.clean_vcf_output(out_file, _clean_varscan_line)
+
+def _clean_varscan_line(line):
+    """Avoid lines with non-GATC bases, ambiguous output bases make GATK unhappy.
+    """
+    if not line.startswith("#"):
+        parts = line.split("\t")
+        alleles = [x.strip() for x in parts[4].split(",")] + [parts[3].strip()]
+        for a in alleles:
+            if len(set(a) - set("GATCgatc")) > 0:
+                return None
+    return line
