@@ -8,25 +8,25 @@ import contextlib
 import copy
 import csv
 import datetime
+import glob
 import itertools
 import os
+import shutil
 import urllib2
-import glob
 
 import yaml
 
 from bcbio import utils
-from bcbio.bam import fastq
-from bcbio.variation.cortex import get_sample_name
+from bcbio.bam import fastq, sample_name
+from bcbio.pipeline import run_info
 from bcbio.workflow.xprize import HelpArgParser
 
-
 def parse_args(inputs):
     parser = HelpArgParser(
         description="Create a bcbio_sample.yaml file from a standard template and inputs")
     parser.add_argument("template", help=("Template name or path to template YAML file. "
-            "Built in choices: freebayes-variant, gatk-variant, tumor-paired, "
-            "illumina-rnaseq, illumina-chipseq"))
+                                          "Built in choices: freebayes-variant, gatk-variant, tumor-paired, "
+                                          "noalign-variant, illumina-rnaseq, illumina-chipseq"))
     parser.add_argument("metadata", help="CSV file with project metadata. Name of file used as project name.")
     parser.add_argument("input_files", nargs="*", help="Input read files, in BAM or fastq format")
     return parser.parse_args(inputs)
@@ -37,8 +37,8 @@ def _prep_bam_input(f, i, base):
     if not os.path.exists(f):
         raise ValueError("Could not find input file: %s" % f)
     cur = copy.deepcopy(base)
-    cur["files"] = os.path.abspath(f)
-    cur["description"] = get_sample_name(f) or os.path.splitext(os.path.basename(f))[0]
+    cur["files"] = [os.path.abspath(f)]
+    cur["description"] = sample_name(f) or os.path.splitext(os.path.basename(f))[0]
     return cur
 
 def _prep_fastq_input(fs, base):
@@ -47,45 +47,42 @@ def _prep_fastq_input(fs, base):
             raise ValueError("Could not find input file: %s" % f)
     cur = copy.deepcopy(base)
     cur["files"] = [os.path.abspath(f) for f in fs]
-    d = os.path.commonprefix([os.path.splitext(os.path.basename(f))[0] for f in fs])
-    to_strip = ("_R", "_", "fastq", ".")
-    while d.endswith(to_strip):
-        for x in to_strip:
-            if d.endswith(x):
-                d = d[:len(d) - len(x)]
-                break
-    cur["description"] = d
+    d = os.path.commonprefix([utils.splitext_plus(os.path.basename(f))[0] for f in fs])
+    cur["description"] = fastq.rstrip_extra(d)
     return cur
 
 def _prep_items_from_base(base, in_files):
     """Prepare a set of configuration items for input files.
     """
     details = []
-    fq_exts = [".fq", ".fastq", ".txt", ".gz"]
-    gz_exts = tuple(["%s.gz" % ext for ext in fq_exts if ext != ".gz"])
-    in_files = _expand_dirs(in_files)
+    known_exts = {".bam": "bam", ".fq": "fastq",
+                  ".fastq": "fastq", ".txt": "fastq",
+                  ".fastq.gz": "fastq", ".fq.gz": "fastq",
+                  ".txt.gz": "fastq", ".gz": "fastq"}
+    in_files = _expand_dirs(in_files, known_exts)
     in_files = _expand_wildcards(in_files)
 
-    for i, (ext, files) in enumerate(itertools.groupby(in_files, lambda x: os.path.splitext(x)[-1].lower())):
-        if ext == ".bam":
+    for i, (ext, files) in enumerate(itertools.groupby(
+            in_files, lambda x: known_exts.get(utils.splitext_plus(x)[-1].lower()))):
+        if ext == "bam":
             for f in files:
                 details.append(_prep_bam_input(f, i, base))
-        elif ext in fq_exts:
+        elif ext == "fastq":
             files = list(files)
-            if ext == ".gz": assert all(f.endswith(gz_exts) for f in files), (files, gz_exts)
             for fs in fastq.combine_pairs(files):
                 details.append(_prep_fastq_input(fs, base))
-        # else:
-        #     raise ValueError("File type not yet implemented: %s" % ext)
+        else:
+            raise ValueError("Unexpected input file types: %s" % str(files))
     return details
 
-def _expand_dirs(in_files):
+def _expand_dirs(in_files, known_exts):
     def _is_dir(in_file):
         return os.path.isdir(os.path.expanduser(in_file))
     files, dirs = utils.partition(_is_dir, in_files)
     for dir in dirs:
-        wildcard = os.path.join(os.path.expanduser(dir), "*")
-        files = itertools.chain(glob.glob(wildcard), files)
+        for ext in known_exts.keys():
+            wildcard = os.path.join(os.path.expanduser(dir), "*"+ext)
+            files = itertools.chain(glob.glob(wildcard), files)
     return list(files)
 
 def _expand_wildcards(in_files):
@@ -98,7 +95,6 @@ def _expand_wildcards(in_files):
         files = itertools.chain(glob.glob(abs_path), files)
     return list(files)
 
-
 # ## Read and write configuration files
 
 def _read_template(template):
@@ -136,13 +132,16 @@ def _write_config_file(items, template, project_name, out_dir):
     """
     config_dir = utils.safe_makedir(os.path.join(out_dir, "config"))
     out_config_file = os.path.join(config_dir, "%s.yaml" % project_name)
-    out = {"fc_date": datetime.datetime.now().strftime("%y%m%d"),
+    out = {"fc_date": datetime.datetime.now().strftime("%Y-%m-%d"),
            "fc_name": project_name,
            "upload": {"dir": "../final"},
            "details": items}
     for k, v in template.iteritems():
         if k not in ["details"]:
             out[k] = v
+    if os.path.exists(out_config_file):
+        shutil.move(out_config_file,
+                    out_config_file + ".bak%s" % datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S"))
     with open(out_config_file, "w") as out_handle:
         yaml.dump(out, out_handle, default_flow_style=False, allow_unicode=False)
     return out_config_file
@@ -191,26 +190,42 @@ def _add_metadata(item, metadata):
     Adds to object and handles special keys:
     - `description`: A new description for the item. Used to relabel items
        based on the pre-determined description from fastq name or BAM read groups.
+    - Keys matching supported names in the algorithm section map
+      to key/value pairs there instead of metadata.
     """
-    item_md = metadata.get(item["description"], {})
-    if "description" in item_md:
-        if item_md["description"]:
-            item["description"] = item_md["description"]
-            del item_md["description"]
+    item_md = metadata.get(item["description"],
+                           metadata.get(os.path.basename(item["files"][0]), {}))
+    TOP_LEVEL = set(["description"])
     if len(item_md) > 0:
         if "metadata" not in item:
             item["metadata"] = {}
         for k, v in item_md.iteritems():
             if v:
-                item["metadata"][k] = v
+                if k in TOP_LEVEL:
+                    item[k] = v
+                elif k in run_info.ALGORITHM_KEYS:
+                    # Handle keys that pass integer or boolean values
+                    try:
+                        v = int(v)
+                    except ValueError:
+                        if v.lower() == "true":
+                            v = True
+                        elif v.lower() == "false":
+                            v = False
+                    item["algorithm"][k] = v
+                else:
+                    item["metadata"][k] = v
+    elif len(metadata) > 0:
+        print "Metadata not found for sample %s, %s" % (item["description"],
+                                                        os.path.basename(item["files"][0]))
     return item
 
 def setup(args):
     template, template_txt = _read_template(args.template)
     base_item = template["details"][0]
     project_name, metadata = _pname_and_metadata(args.metadata)
-    items = [_add_metadata(item, metadata) for item in
-             _prep_items_from_base(base_item, args.input_files)]
+    items = [_add_metadata(item, metadata)
+             for item in _prep_items_from_base(base_item, args.input_files)]
 
     out_dir = os.path.join(os.getcwd(), project_name)
     work_dir = utils.safe_makedir(os.path.join(out_dir, "work"))
@@ -226,5 +241,3 @@ def setup(args):
         print "Edit to finalize and run with:"
         print "  cd %s" % work_dir
         print "  bcbio_nextgen.py ../config/%s" % os.path.basename(out_config_file)
-
-
